/*
 * @copyright		© Copyright 2006-2010, epages GmbH, All Rights Reserved.
 *
 * @module			ep.alert
 *
 * @revision		$Revision: 1.1.2.1 $
 */

jQuery.plugin( 'ep.alert', ['ep.core','ep.ui.dialog','ep.sprite', 'jQuery.core'], function( $, undefined ){

	/*
	 * @dictionary		ep.dict
	 *
	 * @translation		{Notification}
	 *					{Ok}
	 *					{Cancel}
	 */

	// size definitions of the future: xl, 64, 128
	var sizeMap= {
			"s": "Small",
			"m": "Medium",
			"l": "Large"
		},
		typeMap= {
			"notification": "Notification",
			"warning": "Warning",
			"confirmation": "Confirmation",
			"tip": "Tip",
			"question": "Question",
			"error": "Error",
			"positiveresult": "Positiveresult"
		};

	ep.extend({
		alert: function( settings ){
			settings = $.extend( {}, ep.alertSettings, settings );

			var dict = ep.dict('ep.dict'),
				classSize = sizeMap[settings.size] || "Large",
				classType = typeMap[settings.type] || "Notification";

			var dialog = ep('<div>')
				.addClass('ep-uiDialog-alert')
					.append( $('<div class="Message">')
										.addClass(classSize+" "+classType)
										.append(settings.content) );
			options = {
				title:		dict.parse(settings.title),
				modal:		true,
				resizable:	false,
				minWidth:	settings.minWidth || 350,
				buttons:	{},
				close:		function( event ){
					if( settings.close ){
						settings.close(event);
					}

					var originalEvent = event.originalEvent,
						originalTarget = originalEvent ? originalEvent.originalTarget : event.originalTarget,
						buttons = ep(this).uiDialog('option','buttons');

					if (event.keyCode !== $.ui.keyCode.ESCAPE && settings.ok && buttons.ok && originalTarget===buttons.ok[0]) {
						settings.ok(event);
					}
					else if (settings.cancel) {
						settings.cancel(event);
					}
				}
			};

			$.each({
				ok: "{Ok}",
				cancel: "{Cancel}"
			}, function( name, translateName ){

				if( settings[ name ] ){
					options.buttons[name] = {
						text: settings[ name + 'Title' ] || dict.translate( translateName ),
						click: function (event) {
							event.originalTarget = event.target;
							ep(this).uiDialog('close',event);
						}
					};
				}
			});

			return dialog.uiDialog(options);
		},

		alertSettings: {
			title:		'{Notification}',
			type:		'notification',
			size:		'l',
			content:	'',
			ok:			null,
			cancel:		null,
			close:		null
		}
	});

});