/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			ep.ui.richTable
 *
 * @revision		$Revision: 1.1.2.1 $
 */
jQuery.plugin("ep.ui.richTable",
	["ep.core", "jQuery.ui.widget", "jQuery.event.special.load", "ep.fn.contextOrientation", "ep.ui.textedit", "ep.ui.validate"],
function($, undefined){

		// helper methods
	var getRow = function( elem ){
			return elem = elem.is(":checkbox") ? elem.closest("tr") : elem;
		},

		getCheckbox = function( elem ){
			return elem = elem.is(":checkbox") ? elem : elem.find("> td:first-child input[type=checkbox]");
		},

		toggleRow = function( elem, bool ){
			var checkbox = getCheckbox(elem);

			getRow(elem).toggleClass("RowSelected",  bool===undefined ? checkbox.is(":checked") : bool);
		},

		toggleCheckbox = function( elem, bool ){
			var checkbox = getCheckbox(elem);

			checkbox
				.not(":disabled")
				.attr("checked", bool===undefined ? !checkbox.is(":checked") : bool)
				.trigger("change");
		},

		specialHandles = [
			"table.ContentList > thead > tr > *:first-child input[type=checkbox]", 						// select all checkbox
			"table.ContentList > tbody > tr > td:first-child input[type=checkbox]",						// select row checkbox
			"table.ContentList > tbody > tr > td[data-ep-editable]"										// editble cell
		],

		specialHandlesExclude = specialHandles.join(", ") + ", " + specialHandles.join(" *, ") + " *";	// exclude from toggle row selections

	$.widget("ep.ui.RichTable", {
		STR_EDIT_ICON: '<i class="Sprite ico_s_edit"></i>',

		options: {
		//	grossPrice:	"c2",						// format for net/gross prices
			editIcon: 	true,						// indicator if a edit icon should be shown
			alterIcon: 	false,						// flag if the edit icon should be shown always right (false) or depending by align (true)
			region:		$.i18n.settings.region,		// region string
			currency:	$.i18n.settings.currency,	// currency acronym
			saveAction: "Save"						// save method
		},

		_create: function(){
			var self = this,
				o = self.options;

			if(self.element[0]===undefined){
				return;
			}

			// get the currency format (default "c2": two decimal places)
			o.grossPrice = (o.grossPrice===undefined)? "c2" : o.grossPrice;

			self.element
				.addClass("RowSelection")
				.on("change.uiRichTable", specialHandles[0], function( event ){								// delegate checkbox for select all
					var isChecked = $(this).is(":checked");

					self.element
						.find("> tbody > tr > td:first-child input[type=checkbox]")
						.each(function(){
							toggleCheckbox( $(this), isChecked );
						});
				})
				.on("change.uiRichTable", specialHandles[1], function( event ){								// delegate checkbox for row selection
					var input = $(this);

					toggleRow( input );
				})
				.on("mouseenter", specialHandles[2] + ":not(.RowSelectionInputCell)", function(){			// delegate mouseenter editable cell for initalize
					var o = self.options,
						cell = $(this).addClass("RowSelectionInputCell"),
						wrapper = cell.find("*[data-ep-attribute]");

					if( wrapper.length > 0 ){

						wrapper.addClass("epEditWrapper ClickIcon");

						// if an edit icon should be shown
						if( o.editIcon ){
							if( !o.alterIcon || cell.css("text-align") != "right" ){
								wrapper
									.append(self.STR_EDIT_ICON)
									.data("epTextAlign", "left");

								if( cell.css("text-align") == "right" ){
									cell.addClass("ep-Textedit-right");
								}
							}
							else{
								wrapper
									.prepend(self.STR_EDIT_ICON)
									.data("epTextAlign", "right");

								cell.addClass("ep-Textedit-right");
							}
						}

						switch( wrapper.data("epAttribute").toLowerCase() ){
							case "name":
								wrapper.css("display", "inline-block");
								break;
						/*
							// if price and stocklevel wrapper need special formatting later
							case "price":
								break;
							case "stocklevel":
								break;
						*/
						}
					}
				})
				.on("click.uiRichTable", "table.ContentList > tbody > tr", function( event ){				// delegate cell click
					var target = $(event.originalTarget),
						cell = target.closest("td"),
						row = $(this),
						data;

					if( cell.is(specialHandles[2]) ){
						toggleCheckbox( row, false );
						self._handleEditable( row, cell, target );
					}
					else if( !target.is(specialHandlesExclude) ){
						toggleCheckbox( row );
					}
				});
		},

		_handleEditable: function( row, cell, target ){
			var self = this,
				o = self.options,
				wrapper = ep(".epEditWrapper", cell),	// element which should be editable
				oValidate,								// object/hash for the uiValidate widget
				oTextEditOptions;						// object/hash for the uiTextedit widget

			// if the element is not editable yet
			if( wrapper.data('epUiTextedit') === undefined ){

				// mark that the element is just created
				wrapper.prop("justCreated", true);

				switch( wrapper.data("epAttribute") ){
					case "Name":						// value of the data-ep-attribute correlate to the Save attribute
						oValidate = {
							maxlength: 100
						};
						break;
					case "Price":						// value of the data-ep-attribute correlate to the Save attribute
						oValidate = {
							type: "number",
							format: o.grossPrice,
							region: o.region,
							currency: o.currency,
							min: -2147483648,
							max: 2147483647,
							needFormat: true
						};
						break;
					case "StockLevel":					// value of the data-ep-attribute correlate to the Save attribute
						oValidate = {
							type: "number",
							format: "n2",
							region: o.region,
							min: -2147483648,
							max: 2147483647
						};
						break;
					default:
						oValidate = {};
						break;
				}

				oValidate.valid = false;

				// object/hash for the uiTextedit widget
				oTextEditOptions = {
					action		: self.options.saveAction, 						// SaveAction "Save"
					objectId	: getCheckbox(row).val(),						// objectId
					name		: wrapper.data("epAttribute"),					// name/value pair (from data-ep-attribute)
					stripHTML	: true,
					validate	: oValidate,
					preventDefaultOnEnter : true,

					// callback called before element becomes editable
					onBeforeOpen: function(input){
						var icon = wrapper.find(".Sprite"),
							iconWidth;

						// remove edit icon if necessary
						if(self.options.editIcon){
							iconWidth = icon.outerWidth() +  parseInt(icon.css('margin-left'));
							ep(".Sprite", wrapper).remove();
						}
						// show element
						input.show();
						input.attr("disabled", false);

						if(self.options.editIcon && wrapper.parent().hasClass('ep-Textedit-right')){
							input.css('margin-right', iconWidth + 'px');
						}

						wrapper.css("width","98%");
						cell.find("*.hideOnEditable").hide();
					},

					onAfterOpen: function(input){
						// if element was just created
						if( wrapper.prop("justCreated") ){
							wrapper.prop("justCreated", false);
							// prevent an invalid message if input is empty
							if(input.val().length==0){
								input[0].formInvalid = false;
								input.blur();
								wrapper.trigger("click.uiTextedit");
								//input.focus();
							}
						}
					},

					// Called before data gets written to *wrapper*.
					formatOnWriteBack: function(input, evt) {
						// Format input value.
						var value = (oValidate.needFormat===true && oValidate.type=="number" && !isNaN( $.i18n.parseNumber(input.val()))) ?
								$.i18n.formatNumber($.i18n.parseNumber(input.val()), oValidate.format, {currency: self.options.currency, region: self.options.region}) :
								input.val();

						input.val(value);

						if ((typeof evt!="undefined") && evt.keyCode==27) {
						// On "ESC" *onAfterClose* will not be called.
							wrapper.hide();
							var invalid = input.is(":input:invalid");
							// short delay for webkit-engines to prevent default action
							setTimeout(function(){
						 		self._setContent(input, wrapper, true);
								if (!invalid) {
									wrapper.show();
									input.hide();
								}
								else {
									wrapper.hide();
									input.show();
								}
								wrapper.css("width","");
								cell.find("*.hideOnEditable").show();
							},10);
						}
					},

					// Called after element becomes uneditable and data has been saved to server.
					onAfterClose: function(input){
						self._setContent(input, wrapper, true);

						// If HasSubOwnPrices has just been set,
						// update the DOM and the corresponding server action.
						var ownpriceWasSet = wrapper.data("epOwnprice");
						if (ownpriceWasSet !== undefined) {
							// Update style and title of *wrapper* enclosing *cell*.
							cell.removeClass("InheritedValue");
							cell.attr("title", wrapper.data("epOwnpriceTitle"));

							// Update *additional* option of *uiTextedit*.
							var additional = wrapper.uiTextedit("option","additional");
							delete additional[ownpriceWasSet];
							wrapper.uiTextedit("option","additional",additional);

							// Update DOM of *wrapper*.
							wrapper
								.removeData(["epOwnprice","epOwnpriceTitle"])
								.removeAttr("data-ep-ownprice")
								.removeAttr("data-ep-ownprice-title");
						}

						wrapper.css("width","");
						cell.find("*.hideOnEditable").show();
					}
				};

				// if additional parameters are needed for the SaveAction
				if(wrapper.data("epAdditional")!==undefined){
					oTextEditOptions.additional = {};
					// e.g. oTextEditOptions.additional["CurrencyID"] = "EUR"
					oTextEditOptions.additional[wrapper.data("epAdditional")] = self.options.currency;
				}

				// if additional parameters are needed for the SaveAction
				if(wrapper.data("epOwnprice")!==undefined){
					oTextEditOptions.additional = (oTextEditOptions.additional===undefined)? {} : oTextEditOptions.additional;
					// e.g. oTextEditOptions.additional["HasSubOwnPrices"] = 1
					oTextEditOptions.additional[wrapper.data("epOwnprice")] = 1;
				}

				// build the uiTextedit widget
				wrapper.uiTextedit(oTextEditOptions);
			}

			// trigger click event to show the editable element
			wrapper.uiTextedit("open");
		},

		_setContent: function( input, wrapper, disabled ){
			var self = this;

			input.attr("disabled", disabled);

			// if an edit icons should be shown
			if( self.options.editIcon ){
				if( !self.options.alterIcon || wrapper.data("epTextAlign") != "right" ){
					wrapper.append(self.STR_EDIT_ICON);
				}
				else{
					wrapper.prepend(self.STR_EDIT_ICON);
				}
			}
		}
	});
});