use strict;
use Test::More tests => 4;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle);
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    # =============================================================================
    # direct DB access (no test cases, just to see what the driver does by default)
    # =============================================================================
    my $dbi = GetCurrentDBHandle();
    my $DriverName = $dbi->driverName;
    print "# Driver: $DriverName\n";
    my $DataType = $DriverName eq 'Sybase' ? 'MONEY' : 'DECIMAL(10,4)';
    my $TempTableName = $dbi->tempTableName('mydiscount');
    $dbi->createTempTable("mydiscount (value $DataType NOT NULL)");

    $dbi->do("INSERT INTO $TempTableName values (10)");
    my $value = $dbi->execute("SELECT value FROM $TempTableName")->[0]->[0];
    print "# 10 eq $value (natural number)\n";

    $dbi->do("DELETE FROM $TempTableName");
    $dbi->do("INSERT INTO $TempTableName VALUES (10.15)");
    $value = $dbi->execute("SELECT value FROM $TempTableName")->[0]->[0];
    print "# 10.15 eq $value (natural number)\n";

    $dbi->dropTable($TempTableName);


    # ==========================================================================
    # DAL
    # ==========================================================================
    my $Product   = LoadObjectByPath('/Test/TestShop/Products/MyRedNose');
    my $ListPrice = $Product->get('ListPrices')->[0]->{'ListPrice'};
    ok( '8.3' eq $ListPrice, "8.3 eq $ListPrice (real number)" );

    $Product   = LoadObjectByPath('/Test/TestShop/Products/MyYellowNose');
    $ListPrice = $Product->get('ListPrices')->[0]->{'ListPrice'};
    ok( '4' eq $ListPrice, "4 eq $ListPrice (real number)" );

    ok( Cleanup(), 'cleanup' );
    DisconnectSite();
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { 'Alias' => 'Test' } );
    ImportObjects( 'import.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main, Debug => 1 );
